import { select } from 'd3';
import { attachClasses, returnWinner } from './utilities.js';
import { handleClick, handleMouseOver, handleMouseOut } from './events.js';
import { generateLineData } from './generatedata';

// This is an example of an ES6 module.
export function drawChart(data){
  
  console.log('drawChart', data);
  
  data.sort((a, b) => b.population - a.population);
  
  const lineData = generateLineData(data);
  console.log(lineData);
  
  let priorRadius;

  const t = d3.transition()
      .duration(100)
      .ease(d3.easeLinear);
  
  // SET DIMENSIONS ON DESKTOP
  const margin = {left: 50, top: 10, right: 10, bottom: 45};
  const width = $('#chart').width();
  const chartWidth = width - margin.left - margin.right;
  const height = width * 2/3;
  $('#chart').height(height);
  
  const svg = d3.select('#chart').append('svg')
    .attr('width', (width + margin.left + margin.right))
    .attr('height', (height + margin.top + margin.bottom));

  const wrapper = svg.append('g').attr('class', 'chartWrapper')
  			.attr('transform', `translate(${margin.left}, ${margin.top})`);

  // SET THE SCALES
  const xScale = d3.scaleLinear()
  	.range([0, width])
  	.domain([-100,100])
    .nice();
  const yScale = d3.scaleLinear()
  	.range([height-margin.bottom,0])
  	.domain(d3.extent(data, d => d.voter_turnout))
  	.nice();  
  const rScale = d3.scaleLinear()
  	.range([3, 30])
  	.domain(d3.extent(data, d => d.population))
  	.nice();

  // SET THE AXES
  const xAxis = d3.axisBottom()
  	.tickFormat(d3.format(".0%"))
  	.scale(xScale);
  const yAxis = d3.axisLeft()
  	.tickFormat(d3.format(".0%"))
  	.scale(yScale);
  
  // SET ARROWHEAD
  svg.append("defs").append("marker")
    .attr("id", "arrowhead")
    .attr("refY", 4)
    .attr("markerWidth", 12)
    .attr("markerHeight", 12)
    .attr("orient", "auto")
    .append("path")
        .attr("d", "M 0,0 V 8 L12,4 Z"); //this is actual shape for arrowhead
      
  // DRAW LINE
  wrapper.selectAll('line')
    .data(lineData)
    	.enter().append('line')
        .attr('class', d => `web ${d.id}`)
        .attr('opacity', 0.05)
        .attr('x1', d => xScale(d.winner2012))
        .attr('y1', d => yScale(d.turnout2012))
        .attr('x2', d => xScale(d.winner2018))
        .attr('y2', d => yScale(d.turnout2018))
        .attr('marker-end', 'url(#arrowhead)')
  
  d3.selectAll('#chart marker')
  	.attr("refX", 15)
  
  // DRAW THE AXES
  // wrapper.append('g')
  // 	.attr('class', 'x axis')
  // 	.attr('transform', `translate(0, ${height - margin.bottom})`)
  // 	.call(xAxis);
  // wrapper.append('g')
  //   .attr("class", "y axis")
  //   .attr('transform', `translate(0, 0)`)
  //   .call(yAxis);
  
  // DRAW CIRCLES
  wrapper.selectAll('counties')
  	.data(data) //Sort so the biggest circles are below
  	.enter().append('circle')
  		.attr('class', d => attachClasses(d))
  		.attr('cx', d => xScale(returnWinner(d)))
  		.attr('cy', d => yScale(d.voter_turnout))
  		.attr('r', d => rScale(d.population))
      .on('click', handleClick)
      .on('mouseover', handleMouseOver)
      .on('mouseout', handleMouseOut);
  
  // DRAW X-AXIS LABELS
  svg.append('text')
    .attr('class', 'x title')
    .attr('text-anchor', 'end')
    .style('font-size', '12px')
    .attr('transform', `translate(${$('#chart').width()},${height - 5})`)
    .text('% Republican');

  // DRAW X-AXIS LABELS
  svg.append('text')
    .attr('class', 'x title')
    .style('font-size', '12px')
    .attr('transform', `translate(${margin.left},${height - 5})`)
    .text('% Democrat');

  // DRAW Y-AXIS LABEL
  svg.append('g')
  	.append('text')
  	.attr('class', 'y title')
  	.attr('text-anchor', 'end')
  	.style('font-size', '12px')
  	.attr('transform', 'translate(0, 0) rotate(-90)')
  	.attr('alignment-baseline', 'hanging')
  	.text('Voter turnout');
  
}